/*
 * This file is part of the UCB release of Plan 9. It is subject to the license
 * terms in the LICENSE file found in the top-level directory of this
 * distribution and at http://akaros.cs.berkeley.edu/files/Plan9License. No
 * part of the UCB release of Plan 9, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <u.h>
#include <libc.h>

/*
 * Since the SSL device uses decimal file descriptors to name channels,
 * it is impossible for a user-level file server to stand in for the kernel device.
 * Thus we hard-code #D rather than use /net/ssl.
 */

int
jehanne_pushssl(int fd, const char *alg, const char *secin, const char *secout, int *cfd)
{
	char buf[8];
	char dname[64];
	int n, data, ctl;

	ctl = sys_open("#D/ssl/clone", ORDWR);
	if(ctl < 0)
		return -1;
	n = jehanne_read(ctl, buf, sizeof(buf)-1);
	if(n < 0)
		goto error;
	buf[n] = 0;
	jehanne_sprint(dname, "#D/ssl/%s/data", buf);
	data = sys_open(dname, ORDWR);
	if(data < 0)
		goto error;
	if(jehanne_fprint(ctl, "fd %d", fd) < 0 ||
	   jehanne_fprint(ctl, "secretin %s", secin) < 0 ||
	   jehanne_fprint(ctl, "secretout %s", secout) < 0 ||
	   jehanne_fprint(ctl, "alg %s", alg) < 0){
		sys_close(data);
		goto error;
	}
	sys_close(fd);
	if(cfd != 0)
		*cfd = ctl;
	else
		sys_close(ctl);
	return data;
error:
	sys_close(ctl);
	return -1;
}
