/*
 * This file is part of the UCB release of Plan 9. It is subject to the license
 * terms in the LICENSE file found in the top-level directory of this
 * distribution and at http://akaros.cs.berkeley.edu/files/Plan9License. No
 * part of the UCB release of Plan 9, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <u.h>
#include <libc.h>
#include <thread.h>
#include "threadimpl.h"

static int32_t totalmalloc;

void*
_threadmalloc(int32_t size, int z)
{
	void *m;

	m = jehanne_malloc(size);
	if (m == nil)
		jehanne_sysfatal("Malloc of size %ld failed: %r", size);
	jehanne_setmalloctag(m, jehanne_getcallerpc());
	totalmalloc += size;
	if (size > 100000000) {
		jehanne_fprint(2, "Malloc of size %ld, total %ld\n", size, totalmalloc);
		abort();
	}
	if (z)
		jehanne_memset(m, 0, size);
	return m;
}

void
_threadsysfatal(char *fmt, ...)
{
	va_list arg;
	char buf[1024];	/* size doesn't matter; we're about to exit */

	va_start(arg, fmt);
	jehanne_vseprint(buf, buf+sizeof(buf), fmt, arg);
	va_end(arg);
	if(argv0)
		jehanne_fprint(2, "%s: %s\n", argv0, buf);
	else
		jehanne_fprint(2, "%s\n", buf);
	threadexitsall(buf);
}
